#The following code allows the user to recreate the results obtained with the two-stage index

#######
#Step 1: Reading in the data
#######

#First set the working directory
setwd("C:/Users/kvincent/Desktop/MoH PCA project/Kyle's work")
#Read in all the data
Data_All = read.csv("Complete Reduced Database Final.csv",header=TRUE)

#Removing the preamble
Data_sub_All = read.csv("Data Working Version.csv",header=TRUE)
#Data_sub_All contains the former assignment of dissemination areas to local health areas

#Upload the British Columbia Ministry of Health's file
New_LHAs = read.csv("DA_CSD_LHA_2006_Jeet_Rana.csv",header=TRUE)

#The Unused_New_LHAs are those dissemination areas that are not included in our oringial data set and can't be used due to an absence of reported income/census data
#Here are the unused dissemination areas, there are 1,031 of them
Unused_New_LHAs = subset(New_LHAs,New_LHAs[,1] %in% intersect(New_LHAs[,1],Data_sub_All[,2])==FALSE)

#Here are the new assignments for the 6,440 dissemination areas
New_LHAs = subset(New_LHAs,New_LHAs[,1] %in% intersect(New_LHAs[,1],Data_sub_All[,2]))

#Carry the new assignments over to our original census data set
Data_sub_All[,6:7] = New_LHAs[,3:4] #Update the file we work with

#Load libraries
library(fields)  #For drawing liines in R
library(tseries) #For the Jaques-Bera test in R

#Now to sort the data for the analysis
Data_sorted = read.csv("Sort data.csv",header=TRUE,skip=1)
Data_sorted = Data_sorted[,-1]


#####
#Step 2: Running a PCA for each of the six categories
#####

Data = scale(Data_sorted)			# First scale the data, easiest to work with this
X11(); #We will plot the eigenvalues from each domain
par(mfrow=c(2,3))
##Education 
pData.Education=princomp(Data[,1:20],cor=TRUE,center=TRUE,scale=TRUE,cutoff=0) #The preliminary principal components anlaysis (PCA)
plot(pData.Education$sdev^2,typ='h',main="Education")
##Unemployment 
pData.Unemployment=princomp(Data[,21:38],cor=TRUE,center=TRUE,scale=TRUE,cutoff=0) #The preliminary principal components anlaysis (PCA)
plot(pData.Unemployment$sdev^2,typ='h',main="Unemployment")
##Income
pData.Income=princomp(Data[,39:48],cor=TRUE,center=TRUE,scale=TRUE,cutoff=0) #The preliminary principal components anlaysis (PCA)
plot(pData.Income$sdev^2,typ='h',main="Income")
##Housing
pData.Housing=princomp(Data[,49:57],cor=TRUE,center=TRUE,scale=TRUE,cutoff=0) #The preliminary principal components anlaysis (PCA)
plot(pData.Housing$sdev^2,typ='h',main="Housing")
##Occupation
pData.Occupation=princomp(Data[,58:62],cor=TRUE,center=TRUE,scale=TRUE,cutoff=0) #The preliminary principal components anlaysis (PCA)
plot(pData.Occupation$sdev^2,typ='h',main="Occupation")
##Social
pData.Social=princomp(Data[,59:78],cor=TRUE,center=TRUE,scale=TRUE,cutoff=0) #The preliminary principal components anlaysis (PCA)
plot(pData.Social$sdev^2,typ='h',main="Social")
par(mfrow=c(1,1))

#####
#Step 3: Running a PCA on the Principal Components, we have retained the first principal component from each pca for this analysis
#####
Data = cbind(pData.Education$scores[,1],pData.Unemployment$scores[,1],pData.Income$scores[,1],pData.Housing$scores[,1],pData.Occupation$scores[,1],pData.Social$scores[,1])

pData=princomp(Data,cor=TRUE,center=TRUE,scale=TRUE,cutoff=0,retx=TRUE)	
weights.pca.1 = -1*loadings(pData)[,1] #The index values, we multiply by -1 to preserve order, high scores imply more deprived areas
weights.pca.2 = loadings(pData)[,2] #A second index, may provide some insight into a material vs. social deprivation
#Compare these two to see a potential contrast 
weights.pca.1
weights.pca.2


#######
#Step 4: Obtaining scores for the dissemination areas
#######

Score.two_stage = numeric(6440)
for(k in 1:6440)
	Score.two_stage[k] = sum(Data[k,] * weights.pca.1)
	
#Standardize these values
Score.two_stage = (Score.two_stage - mean(Score.two_stage)) / sd(Score.two_stage)
Score.two_stage = 100*Score.two_stage + 1000

X11();	
hist(Score.two_stage,col="lightblue",breaks=100,xlim=range(550,1450))	#A histogram to see the scores

#Test for Normality, the Jaque Bera test depends on skewness and kurtosis. However, this is a very large sample, therefore the QQ plot is strongly suggested.
jarque.bera.test(Score.two_stage)

qqnorm(Score.two_stage)				#A QQ-plot

#The 5 dissemination areas that received the highest scores with scores provided
Data_sub_All[which(Score.two_stage>1315.1),1:7]
Score.two_stage[which(Score.two_stage>1315.1)]

#The 5 dissemination areas that received the lowest scores with scores provided
Data_sub_All[which(Score.two_stage<605),1:7]
Score.two_stage[which(Score.two_stage<605)]


#######
#Step 5: Obtaining scores for the local health areas
#######

#Append the scores from dissemination areas to the updated data set
Data_Score.two_stage = cbind(Data_sub_All[,1:7],Score.two_stage)
num_var = dim(Data_Score.two_stage)[1]
LHA_Score.two_stage = numeric(length(levels(Data_Score.two_stage[,7])))

jj = 0
for(k in levels(Data_Score.two_stage[,7]))
	{
	score.temp = numeric()
	j = 0
	for(i in 1:num_var)
		{
		if(Data_Score.two_stage[i,7] == k)
			{
			j = j+1
			score.temp[j] = Data_Score.two_stage[i,8]
			}
		}
	jj = jj+1
	LHA_Score.two_stage[jj] = mean(score.temp)	
	}

LHA_Score_estimates = LHA_Score.two_stage	#LHA_Score_estimates will be used at the bootstrap stage of the analysis

LHA_Score.two_stage = cbind(LHA_Score.two_stage,levels(Data_Score.two_stage[,7])) #The final scores, can be compared against with draft paper, page 46
LHA_Score.two_stage #First by ALPHABETICAL order

num_var=dim(LHA_Score.two_stage)[1]
scores = as.numeric(LHA_Score.two_stage[,1])

for(k in 1:length(scores))
	if(is.finite(scores[k]) == FALSE)
		scores[k] = 0
		

LHA_Score.two_stage=LHA_Score.two_stage[sort(scores,index.return=TRUE)$ix,]
LHA_Score.two_stage #Now by ASCENDING order
	

################################	
#Recreating the prototype index
#This is the work based on the prototype index and can be found in Regional Measures (2012)
#The work that is recreated here uses the new assignemts for the dissemination areas
###############################

#Working with the necessary data
Data = Data_sub_All[,c(5,8:84)]

#Data Reduction
Data = subset(Data,select=-c(POP15,POPALL))	#First remove the population measures

num_var = dim(Data)[2]						#Now remove those variables with a median of zero
medians = numeric()
for(k in 1:num_var)
	if(median(Data[,k]) == 0)
		medians = c(medians,k)

Data = Data[,setdiff(1:num_var,medians)]

#The following variables were those that were retained for the principal variable model
prototype = numeric()
prototype[1]=which(colnames(Data)==c("INCMEH"))
prototype[2]=which(colnames(Data)==c("EDNO65"))
prototype[3]=which(colnames(Data)==c("OCCONE"))
prototype[4]=which(colnames(Data)==c("UNEMP"))
prototype[5]=which(colnames(Data)==c("EDCO65"))
prototype[6]=which(colnames(Data)==c("EDCO25"))
prototype[7]=which(colnames(Data)==c("NMTONG"))
prototype[8]=which(colnames(Data)==c("EDHS65"))
prototype[9]=which(colnames(Data)==c("EDCO15"))
prototype[10]=which(colnames(Data)==c("EDHS15"))
prototype[11]=which(colnames(Data)==c("EDAP65"))
prototype[12]=which(colnames(Data)==c("PART15"))
prototype


#Now the loadings from the first pc for the weights
Data = Data[,prototype]
Data = scale(Data)			#Scale the data
pData=princomp(Data,cor=TRUE,center=TRUE,scale=TRUE,cutoff=0,retx=TRUE)
weights.pca.1 = -1*loadings(pData)[,1]
weights.pca.2 = -1*loadings(pData)[,2]
#Compare these two to see a potential contrast in material and social deprivation
weights.pca.1
weights.pca.2

#Now the scores for the 6440 dissemination areas, based on the first index
Score.prototype = numeric(6440)
for(k in 1:6440)
	Score.prototype[k] = sum(Data[k,] * weights.pca.1)
	
#Standardize these values
Score.prototype = (Score.prototype - mean(Score.prototype)) / sd(Score.prototype)
Score.prototype = 100*Score.prototype + 1000

hist(Score.prototype,col="lightblue",breaks=100,xlim=range(550,1450)) 

#Test for Normality
jarque.bera.test(Score.prototype)	

qqnorm(Score.prototype)			#Providing a qq plot


#Now to score over the LHAs
#This step will take up to 2 minutes to complete
#These would be the prototype's final scores based on the updated list of the assignment of dissemination areas to LHAs
Data_Score.prototype = cbind(Data_sub_All[,1:7],Score.prototype)

num_var = dim(Data_Score.prototype)[1]
LHA_Score.prototype = numeric(length(levels(Data_Score.prototype[,7])))

jj = 0
for(k in levels(Data_Score.prototype[,7]))
	{
	score.temp = numeric()
	j = 0
	for(i in 1:num_var)
		{
		if(Data_Score.prototype[i,7] == k)
			{
			j = j+1
			score.temp[j] = Data_Score.prototype[i,8]
			}
		}
	jj = jj+1
	LHA_Score.prototype[jj] = mean(score.temp)	
	}

LHA_Score.prototype = cbind(LHA_Score.prototype,levels(Data_Score.prototype[,7])) 
LHA_Score.prototype #First by ALPHABETICAL order

num_var=dim(LHA_Score.prototype)[1]
scores = as.numeric(LHA_Score.prototype[,1])
for(k in 1:length(scores))
	if(is.finite(scores[k])==FALSE)
		scores[k] = 0

LHA_Score.prototype=LHA_Score.prototype[sort(scores,index.return=TRUE)$ix,]
LHA_Score.prototype #Now by NUMERIC order
	

	


##########
#Validation of the two stage index
#########

#####
#Validation 1: The correlation of scores between the two stage and prototype index
cor(Score.two_stage,Score.prototype,method="spearman")	
#####

#####
#Validation 2: The Spearman correlation of the two stage index scores and six proxy variables
#####
#Working with the necessary data, these are the six variables of interest
Data = Data_sub_All[,c(5,8:84)]
Data = scale(Data)
prototype = numeric()
prototype[1]=which(colnames(Data)==c("EDNO25"))
prototype[2]=which(colnames(Data)==c("UNEMP"))
prototype[3]=which(colnames(Data)==c("INCAV"))
prototype[4]=which(colnames(Data)==c("AVVAL"))
prototype[5]=which(colnames(Data)==c("OCCFM"))
prototype[6]=which(colnames(Data)==c("LONEPF"))

#The two_stage index
cor(Score.two_stage,Data[,prototype[1]],method="spearman")
cor(Score.two_stage,Data[,prototype[2]],method="spearman")
cor(Score.two_stage,Data[,prototype[3]],method="spearman")
cor(Score.two_stage,Data[,prototype[4]],method="spearman")
cor(Score.two_stage,Data[,prototype[5]],method="spearman")
cor(Score.two_stage,Data[,prototype[6]],method="spearman")
#The prototype index
cor(Score.prototype,Data[,prototype[1]],method=c("spearman"))
cor(Score.prototype,Data[,prototype[2]],method=c("spearman"))
cor(Score.prototype,Data[,prototype[3]],,method=c("spearman"))
cor(Score.prototype,Data[,prototype[4]],method=c("spearman"))
cor(Score.prototype,Data[,prototype[5]],method=c("spearman"))
cor(Score.prototype,Data[,prototype[6]],method=c("spearman"))




#####
#Validation 3: The INSPQ
####

#First set the working directory to where the Copy of EquivalenceTableCanada2006_A.csv table is:
setwd("C:/Users/kvincent/Desktop/MoH PCA project/MoH files/UBC Files/INSPQ")
INSPQ = read.csv("Copy of EquivalenceTableCanada2006_A.csv",header=TRUE)
INSPQ = INSPQ[46952:54422,]	#The relevant data
INSPQ = na.omit(INSPQ[,c(1,8,9)])	#Keep the first and second principal component scores

DA.Q = intersect(Data_Score.two_stage[,2],INSPQ[,1]) #Which dissemination areas do we have INSPQ scores for?

Score.Q.two_stage = numeric()
Score.Q.prototype = numeric()
INSPQ.M = numeric()
INSPQ.S = numeric()


for(k in DA.Q)
	{
	j = which(INSPQ[,1]==k)
			INSPQ.M = c(INSPQ.M,INSPQ[j,2])
			INSPQ.S = c(INSPQ.S,INSPQ[j,3])
	j = which(Data_Score.two_stage[,2]==k)
			Score.Q.two_stage = c(Score.Q.two_stage,Data_Score.two_stage[j,8])
	j = which(Data_Score.prototype[,2]==k)
			Score.Q.prototype = c(Score.Q.prototype,Data_Score.prototype[j,8])
	}
	
#two stage index score
cor(INSPQ.M,Score.Q.two_stage,method=c("spearman"))
cor(INSPQ.S,Score.Q.two_stage,method=c("spearman"))

#prototype index scores
cor(INSPQ.M,Score.Q.prototype,method=c("spearman"))
cor(INSPQ.S,Score.Q.prototype,method=c("spearman"))
	

#####
#Validation 4: The pseudo-SEFI-like index
#####
#Working with the necessary data
Data = Data_sub_All[,c(5,8:84)]

#The variables of interest
SEFI.variables = numeric()
SEFI.variables[1]=which(colnames(Data)==c("X65PLUS"))
SEFI.variables[2]=which(colnames(Data)==c("POP15"))
SEFI.variables[3]=which(colnames(Data)==c("LONEPM"))
SEFI.variables[4]=which(colnames(Data)==c("LONEPF"))
SEFI.variables[5]=which(colnames(Data)==c("UNEMPF"))
SEFI.variables[6]=which(colnames(Data)==c("UNEMP15"))
SEFI.variables[7]=which(colnames(Data)==c("EDNO25"))

#Combing the related variables
Data.1 = Data[,SEFI.variables[1]]/(Data[,SEFI.variables[1]]+Data[,SEFI.variables[2]])
Data.2 = Data[,SEFI.variables[3]]+Data[,SEFI.variables[4]]
Data.3 = Data[,SEFI.variables[4]]
Data.4 = Data[,SEFI.variables[5]]
Data.5 = Data[,SEFI.variables[6]]
Data.6 = Data[,SEFI.variables[7]]

Data = cbind(Data.1,Data.2,Data.3,Data.4,Data.5,Data.6)

colnames(Data)=c("X65PLUS/(X65PLUS+POP15)","(LONEPM+LONEPF)","LONEPF","UNEMPF","UNEMP15","EDNO25")

Data = scale(Data)

#The first principal component
pData=princomp(Data,cor=TRUE,center=TRUE,scale=TRUE,cutoff=0,retx=TRUE)
weights.pca.1 = -1*loadings(pData)[,1]
weights.pca.1

#Now the Scores for the 6440 dissemination areas
Score.SEFI = numeric(6440)
for(k in 1:6440)
	Score.SEFI[k] = sum(Data[k,] * weights.pca.1)
	
#Standardize these values
Score.SEFI = (Score.SEFI - mean(Score.SEFI)) / sd(Score.SEFI)
Score.SEFI = 100*Score.SEFI + 1000

#The correlation of the two_stage and SEFI index
cor(Score.two_stage,Score.SEFI,method="spearman")

#The correlation of the prototype and SEFI index
cor(Score.prototype,Score.SEFI,method="spearman")

	
	
	
#####################################	
#THE BOOTSTRAP ANALYSIS
####################################

#First set the working directory
setwd("C:/Users/kvincent/Desktop/MoH PCA project/Kyle's work")
write.data = TRUE	#For recording the results
B = 0		#The number of bootstraps

count = numeric()			#Will count the number of dissemination areas in each LHA
num_var = dim(Data_sub_All)[1] #The number of variables to consider

#First determine how many dissemination areas are assigned to each LHA
jj = 0
for(k in levels(Data_sub_All[,7]))
	{
	j = 0
	for(i in 1:num_var)
		if(Data_sub_All[i,7] == k)
			j = j+1
	jj = jj+1
	count[jj] = j
	}	

	
LHA_BOOTSTRAP_SCORES = matrix(0,89,B)	

if(B > 0)
for(kkk in 1:B)
	{

	Data_boot = numeric()

	#The resampling procedure
	for(k in 1:length(count))
		{
		if(count[k] == 1)
			s = which(Data_sub_All[,7]==levels(Data_sub_All[,7])[k])
		else
			s = sample(which(Data_sub_All[,7]==levels(Data_sub_All[,7])[k]),count[k],replace=TRUE)
		Data_boot = rbind(Data_boot,cbind(Data_sub_All[s,7],Data_sorted[s,]))
		}

	Data_bootstrap = scale(Data_boot[-1])			# First scale the data, easiest to work with this

	#Now the first pca on each of the six categories
	##Education 
	pData.Education=princomp(Data_bootstrap[,1:20],cor=TRUE,center=TRUE,scale=TRUE,cutoff=0) #The preliminary principal components anlaysis (PCA)
	##Unemployment 
	pData.Unemployment=princomp(Data_bootstrap[,21:38],cor=TRUE,center=TRUE,scale=TRUE,cutoff=0) #The preliminary principal components anlaysis (PCA)
	##Income
	pData.Income=princomp(Data_bootstrap[,39:48],cor=TRUE,center=TRUE,scale=TRUE,cutoff=0) #The preliminary principal components anlaysis (PCA)
	##Housing
	pData.Housing=princomp(Data_bootstrap[,49:57],cor=TRUE,center=TRUE,scale=TRUE,cutoff=0) #The preliminary principal components anlaysis (PCA)
	##Occupation
	pData.Occupation=princomp(Data_bootstrap[,58:62],cor=TRUE,center=TRUE,scale=TRUE,cutoff=0) #The preliminary principal components anlaysis (PCA)
	##Social
	pData.Social=princomp(Data_bootstrap[,59:78],cor=TRUE,center=TRUE,scale=TRUE,cutoff=0) #The preliminary principal components anlaysis (PCA)

	#The final pca on the first principal components
	Data = cbind(pData.Education$scores[,1],pData.Unemployment$scores[,1],pData.Income$scores[,1],pData.Housing$scores[,1],pData.Occupation$scores[,1],pData.Social$scores[,1])
	pData=princomp(Data,cor=TRUE,center=TRUE,scale=TRUE,cutoff=0,retx=TRUE)		#The PCA
	weights.pca.1 = -1*loadings(pData)[,1] #The index values

	#Now the bootstrap scores for the 6440 dissemination areas
	Score.two_stage = numeric(6440)
	for(k in 1:6440)
		Score.two_stage[k] = sum(Data[k,] * weights.pca.1)
		
	#Standardize these values
	Score.two_stage = (Score.two_stage - mean(Score.two_stage)) / sd(Score.two_stage)
	Score.two_stage = 100*Score.two_stage + 1000

	#Scoring over the LHAs
	num_var = dim(Data_boot)[1]
	LHA_Score = numeric()

	jj = 0
	for(k in levels(Data_sub_All[,7]))
		{
		score.temp = numeric()
		j = 0
		for(i in 1:num_var)
			{
			if(Data_boot[i,1] == k)
				{
				j = j+1
				score.temp[j] = Score.two_stage[i]
				}
			}
		jj = jj+1
		LHA_Score[jj] = mean(score.temp)	
		}
	
	if(write.data == TRUE)
		{
		xx = LHA_Score
			write(x = xx, ncolumns = length(xx), file = "bootstrap_updated", append = TRUE)	
		}
	
	}

######	
#Visualizing the results	
#####

#First set the working directory	
setwd("C:/Users/kvincent/Desktop/MoH PCA project/Kyle's work")

#First by ALPHABETICAL order
y=1:89
plot(LHA_Score_estimates[1:89],y,xlim=range(750,1200),main="Confidence Intervals based on bootstrap analysis",xlab="Deprivation Score",ylab="LHA, alphabetical order")
bootstrap = read.table("bootstrap_updated")	#Read in the table that holds the scores

bootstrap[,13] = bootstrap[,51] = bootstrap[,80] = 0	#These scores are zero as they are affiliated with the three LHAs that do not receive scores

ci = matrix(0,89,2)	#The confidence intervals
for(k in 1:89)
	{
	ci[k,] = quantile(bootstrap[,k],c(0.025,0.975))
	points(c(ci[k,1],ci[k,2]),c(y[k],y[k]),typ='l',col="blue")
	}

	
#Second by ASCENDING order, we will remove the NaN's here	
LHA_Score_estimates[c(13,51,80)] = 0
sorted = sort(LHA_Score_estimates,index.return=TRUE)$ix

plot(LHA_Score_estimates[sorted],1:89,xlim=range(750,1200),main="Confidence Intervals based on bootstrap analysis",xlab="Deprivation Score",ylab="LHA, sorted by score")
j = 0
for(k in sorted)
	{
	j = j+1
	points(c(ci[k,1],ci[k,2]),c(y[j],y[j]),typ='l',col="blue")
	}

	
#Visulaize more results	
count.length = matrix(0,89,2)
for (k in 1:89)
	{
	count.length[k,1] = count[k]
	count.length[k,2] = sd(bootstrap[,k])
	}

#Plot of deviation of bootstrapped scores by number of DAs
plot(count.length,main="Plot of SD of Bootstrapped scores by number of DAs assigend to LHAs",xlab="Number of DAs",ylab="SD")
	
	
#The bootstrap confidence interval scores
cbind(LHA_Score.two_stage,ci[sorted,1],ci[sorted,2])
